# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.

require "thor/group"

module Tomte::Cli::Protocol::Endpoint
  extend Tomte::Cli::Plugin


  define_tasks do

    desc "endpoint", "Boots up the Tomte console with an endpoint for [uri]"
    method_option :environment, :default => "development"
    method_option :debug, :default => false
    method_option "log-level", :default => nil
    method_option "trace-protocol", :default => false
    method_option "enveloper", :default => :null

    def endpoint(uri = 'amqp://localhost')

      try_require "tomte/protocol"
      prepare :endpoint

      @envelopers = {
          :generic => Tomte::Protocol::Enveloper::Generic,
          :null    => Tomte::Protocol::Enveloper::Null,
          :none    => nil
      }.freeze

      if options['trace-protocol']
        puts "-  set tracing on.".important!
        Tomte::Config[:protocol][:trace] = true
      end

      if options['enveloper']
        en  = options['enveloper'].to_sym
        raise ArgumentError, "Unknown enveloper ''#{en}'.'" unless @envelopers.has_key?(en)
        puts "-  set enveloper to '#{en}'."
        @enveloper_class = @envelopers[en]
      end

      ARGV.clear

      try_require "tomte"
      puts "=> Loading #{Tomte.env} console (Tomte v.#{Tomte::VERSION})".blue.bold
      puts "=> connecting to an AMQP bus at '#{uri}'".blue
      ep = Tomte::Protocol::Endpoint.new(:sync => true,
                                         :uri => uri,
                                         :enveloper => @enveloper_class ? @enveloper_class.new : nil).connect!

      puts "- Connected to broker".yellow

      ::Pry.config.hooks.add_hook(:after_session, :disconnect_endpoint) do
        ep.disconnect!
      end

      tomte_console.start ep

    end


  end
end