
# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.


module Tomte
  class Message
    include Tomte::Document

    class Security
      include Tomte::Document

    end

    class Metadata
      include Tomte::Document

      key :routing_key,       String
      key :persistent,        Boolean, :default => false
      key :immediate,         Boolean, :default => false
      key :mandatory,         Boolean, :default => false
      key :user_id,           String

      key :type,              String,  :default => 'tomte/document'
      key :content_type,      String
      key :content_encoding,  String
      key :priority,          Integer
      key :message_id,        String
      key :reply_to,          String
      key :correlation_id,    String
      key :app_id,            String
      key :timestamp,         Integer
      key :expiration,        String

      def to_wire
        Hash.new.tap do |attrs|
          keys.select { |name, key|
            !self[key.name].nil? && (key.name.first != '_') }.each do |name, key|
            value = key.set(self[key.name])
            attrs[name.to_sym] = value
          end
        end
      end

    end

    key :metadata, Tomte::Message::Metadata,  :default => lambda { Tomte::Message::Metadata.new }
    key :security, Tomte::Message::Security,  :default => lambda { Tomte::Message::Security.new }
    key :headers,  Tomte::OpenDocument,       :default => lambda { Tomte::OpenDocument.new },
                                              :cast => false
    key :payload

    class Exception < Message

      class Payload
        include Tomte::Document

        key :exception,       String
        key :message,         String,  :default => ""
        key :backtrace,       Array,   :default => []
        key :attributes,      Array,   :default => []

      end

      key :payload, Tomte::Message::Exception::Payload,
          :default => lambda { Tomte::Message::Exception::Payload.new }


    end

  end

end