# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.


module Tomte::Protocol
  class Channel < AMQP::Channel

    attr_accessor :enveloper

    def initialize(connection = nil, id = AMQP::Channel.next_channel_id, options = {}, &block)

      # allow either an AMQP connection or a Tomte::Endpoint to be given
      connection = connection.async.client if connection.is_a? Endpoint::Adapter

      @enveloper = (options && options.delete(:enveloper))
      super
    end



    # TODO FIXME, fixes a bug upstream
    #
    # https://github.com/ruby-amqp/amqp/commit/80a86610de58699332ccd478a3b4dd66139131e6
    #
    def auto_recover
      return unless auto_recovering?

      self.open do
        @channel_is_open_deferrable.succeed

        # exchanges must be recovered first because queue recovery includes recovery of bindings. MK.
        @exchanges.each { |name, e| e.auto_recover }
        @queues.each    { |name, q| q.auto_recover }
      end
    end # auto_recover


    def exchange(name, type = :direct, opts = {}, &block)
       if exchange = find_exchange(name)
         extended_opts = Exchange.add_default_options(type, name, opts, block)

         validate_parameters_match!(exchange, extended_opts)

         block.call(exchange) if block
         exchange
       else
         register_exchange(Exchange.new(self, type, name, opts, &block))
       end
     end


    def direct(name = 'amq.direct', opts = {}, &block)
      if exchange = find_exchange(name)
        extended_opts = Exchange.add_default_options(:direct, name, opts, block)

        validate_parameters_match!(exchange, extended_opts)

        block.call(exchange) if block
        exchange
      else
        register_exchange(Exchange.new(self, :direct, name, opts, &block))
      end
    end

    def default_exchange
      Exchange.default(self)
    end

    def fanout(name = 'amq.fanout', opts = {}, &block)
      if exchange = find_exchange(name)
        extended_opts = Exchange.add_default_options(:fanout, name, opts, block)

        validate_parameters_match!(exchange, extended_opts)

        block.call(exchange) if block
        exchange
      else
        register_exchange(Exchange.new(self, :fanout, name, opts, &block))
      end
    end

    def topic(name = 'amq.topic', opts = {}, &block)
      if exchange = find_exchange(name)
        extended_opts = Exchange.add_default_options(:topic, name, opts, block)

        validate_parameters_match!(exchange, extended_opts)

        block.call(exchange) if block
        exchange
      else
        register_exchange(Exchange.new(self, :topic, name, opts, &block))
      end
    end

    def headers(name = 'amq.match', opts = {}, &block)
      if exchange = find_exchange(name)
        extended_opts = Exchange.add_default_options(:headers, name, opts, block)

        validate_parameters_match!(exchange, extended_opts)

        block.call(exchange) if block
        exchange
      else
        register_exchange(Exchange.new(self, :headers, name, opts, &block))
      end
    end

    def queue!(name, opts = {}, &block)
      queue = if block.nil?
                Queue.new(self, name, opts)
              else
                shim = Proc.new { |q, method|
                  queue = find_queue(method.queue)
                  if block.arity == 1
                    block.call(queue)
                  else
                    block.call(queue, method.consumer_count, method.message_count)
                  end
                }
                Queue.new(self, name, opts, &shim)
              end

      register_queue(queue)
    end

    # sort of a monkeypatch, for a bug in the AMQP library
    # nowait is handled as a persistent property of the exchange
    # while should be local to the method, so, don't compare it
    #
    def validate_parameters_match!(entity, parameters)
      parameters.delete(:nowait)
      super
    end

  end
end


