# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.


module Tomte::Protocol
  module Dialog
    class Client < Tomte::Protocol::Producer


      Response = Struct.new(:metadata, :payload, :original_payload)

      class Request
        include EM::Deferrable

        attr_reader   :client
        attr_reader   :message_id
        attr_reader   :payload
        attr_accessor :response

        attr_accessor :headers
        attr_reader   :updates

        def replyable?
          !@return_callback.nil?
        end

        def initialize(client, payload, params = {}, &block)

          @dlg_options = {}
          @headers = {'tomte-dlg-options' => @dlg_options }

          @client = client
          @message_id = params[:message_id] || "tomte.dlg.msg.#{Tomte.random_id}"
          @return_callback = block
          @payload = payload || {}

          callback do
            return_answer
          end

          errback do |cause|
            #
            return_answer
          end

        end

        def updates=(cb)
          unless cb.blank?
            @dlg_options[:updates] = true
            @updates = cb
          end
        end

        def received_update(metadata, payload)
          return unless @updates
          (updates.arity == 1) ? updates.call(payload) : updates.call(metadata, payload)
        end

        def returned_with_format_exception(metadata, payload, exception)

          @response = Response.new(metadata, Tomte::Protocol::FormatException.new(
            'An answer has returned but we cannot decode it',
            :metadata => metadata.attributes,
            :format_exception => exception,
            :payload => payload
          ))

          fail :format_exception
        end


        def returned_with_failure(metadata, payload, message)
          @response = Response.new(metadata, Tomte::Protocol::TransportFailure.new(
              'The message has been returned because of a transport exception',
              :transport_metadata => metadata.properties,
              :transport_exception => message ))

          fail :transport_failure
        end

        def returned_with_publish_error(error)
          @response = Response.new(nil, error)
          fail :publish_error
        end

        def returned(metadata, payload)

          if remote_error?(metadata)

             @response = Response.new(metadata, Tomte::Protocol::RemoteException.new(
                 "A remote exception occurred",
                 :remote => metadata.headers['tomte-dialog-exception'] ), payload)

            fail :remote_exception
          else
            @response = Response.new(metadata, payload)
            succeed
          end
        end

        def remote_error?(metadata)
          metadata.headers['tomte-dialog-exception']
        end

        def close

        end

        private
        #######

        def return_answer

          @response ||= Response.new(nil, Tomte::Protocol::Timeout.new)

          begin
            @return_callback.call(@return_callback.arity == 1 ?
                                 response :
                                 [response.metadata, response.payload]) if @return_callback
          rescue Exception => e
            client.log.exception e
          end

        ensure
          close
        end

      end
    end
  end
end
