
# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.


# = this is what this is supposed to do
#
# The enveloper takes
#
#  - an object
#    - any ruby object that can be serialized in the requested serializer
#    - a Tomte::Document object
#    - a Tomte::Message
#
# and optionally
#
#  - hash of options
#
# and produces:
#
#  - a blob that will go in the AMQP payload, which is serialized according to the
#    serializer, or a special method in the payload
#
#  - a set of AMQP headers
#  - a set of AMQP metadata, to be passed to the publish() method
#


module Tomte::Protocol

  module Enveloper

    WireResponse = Struct.new(:body, :metadata, :payload)
    LocalResponse = Struct.new(:payload, :metadata, :message)

    autoload :Utils,    "tomte/protocol/enveloper/utils"
    autoload :Hooks,    "tomte/protocol/enveloper/hooks"
    autoload :Exchange, "tomte/protocol/enveloper/exchange"
    autoload :Queue,    "tomte/protocol/enveloper/queue"
    autoload :Consumer, "tomte/protocol/enveloper/consumer"

    autoload :Null,     "tomte/protocol/enveloper/null"
    autoload :Generic,  "tomte/protocol/enveloper/generic"


    class Error < StandardError; end
    class TypeError < Error; end
  end
end