module Tomte::Protocol
  module RPC

    class Client < Tomte::Protocol::Dialog::Client

      DEFAULT_CAPABILITIES  = [].freeze
      DEFAULT_REQUEST_FLAGS = [].freeze

      def initialize(connection, options = {})

        options[:exchange] = options[:api] if options[:api]

        @client_capabilities   ||= options.fetch(:capabilities, DEFAULT_CAPABILITIES.dup)
        @default_request_flags ||= options.fetch(:flags, DEFAULT_REQUEST_FLAGS.dup)

        if options[:human]
          @client_capabilities.push(:human).sort!.uniq!
          @default_request_flags.push(:human).sort!.uniq!
        end

        super
        @raw ||= options.fetch(:raw, false)
        @enveloper = Tomte::Protocol::Enveloper::Generic.new
      end

      # we choose an enveloper
      def enveloper=(*_); end

      #
      # client.do(:method, a, b, c) do
      #
      # end
      #
      def do(meth, args = {}, &blk)
        call(meth, :args => args, &blk)
      end


      def call(verb, options = {},  &block)
        raise(Error, "RPC called without a block?") unless block_given?

        args        = options.delete(:args)
        raw_request = options.delete(:raw)

        flags = (@default_request_flags + (options[:flags] || [])).uniq.sort!
        flags.push(:human).sort!.uniq! if options[:human]

        request = Request.new(:verb => verb, :parameters => args, :capabilities => @client_capabilities, :flags => flags)


        super(request, options) do |res|

          metadata, response = res.metadata, res.payload

          ##
          if response.is_a?(::Exception) &&
              !res.original_payload.blank? &&
              res.original_payload.exception.is_a?(Exception)

            response =
                Tomte::Protocol::RPCRemoteException.new("A remote exception occurred during the RPC request",
                                                        :remote => res.original_payload.exception)
          end

          if(@raw || raw_request)
            block.call(metadata, response)
          else
            response = response.respond_to?(:response) ? response.response : response
            response = HashWithIndifferentAccess.new(response) if response.is_a?(Hash)

            block.call response
          end
        end
      end

      def proxy
        Tomte::Protocol::RPC::Proxy.new(self)
      end

      private
      #######
    end
  end
end
