
# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.

module Tomte::Protocol::Sync::RPC

  class Proxy
    include Tomte::Protocol::Synchronization

    def initialize(client)
      @__client = client
    end

    def method_missing(verb, *args)
      resync do |wc|
        errback = proc { |exception| wc.return_raise exception }

         @__client.entity.call(verb, :args => args, :errback => errback) do |response|

  
          if response.is_a?(Tomte::Protocol::RPCRemoteException) && response.remote.is_a?(Exception)
            wc.return_raise(response.remote)
          elsif response.is_a?(Exception)
            wc.return_raise(response)
          else
            wc.return(response)
          end
        end
      end
    end
  end

  class Client
    include Tomte::Protocol::Synchronization
    include Tomte::Protocol::Sync::Entity

    attr_reader :entity

    def proxy
      Proxy.new(self)
    end

    def initialize(connection, options = {})

      initialize_from_async_entity connection
      @entity ||= Tomte::Protocol::RPC::Client.new(connection, options)
    end

    def do(meth, args = {})
      call(meth, :args => args)
    end


    def call(verb, options = {})
      resync do |wc|
        errback = proc { |exception| wc.return_raise exception }

        @entity.call(verb, {:errback => errback}.merge(options)) do |response|

          wc.return_raise(response) if response.is_a?(Exception)
          wc.return(response)
        end
      end
    end

  end
end